package data_manage

import (
	"eta/eta_task/utils"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"strings"
)

var (
	EdbDataTableNameMap     map[int]string        // 指标来源对应数据表名
	EdbDataRefreshMethodMap map[int]string        // 指标来源对应的刷新指标方法
	EdbTableNameSourceMap   map[string]*EdbSource // 数据表名对应的指标来源
	EdbSourceIdMap          map[int]*EdbSource    // 指标来源
	EdbSourceExtendIdMap    map[string]int        // 指标来源字符串对应来源ID
)

// EdbSource 指标来源表
type EdbSource struct {
	EdbSourceId      int    `orm:"column(edb_source_id);pk"`
	SourceName       string `description:"指标来源名称"`
	TableName        string `description:"数据表名"`
	EdbAddMethod     string `description:"指标新增接口"`
	EdbRefreshMethod string `description:"指标刷新接口"`
	IsBase           int    `description:"是否为基础指标: 0-否; 1-是"`
	FromBridge       int    `description:"是否来源于桥接服务: 0-否; 1-是"`
	BridgeFlag       string `description:"桥接服务对象标识"`
	SourceExtend     string `description:"扩展字段做查询用"`
}

// GetEdbSourceItemsByCondition 获取指标来源列表
func GetEdbSourceItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*EdbSource, err error) {
	o := orm.NewOrmUsingDB("data")
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := `ORDER BY edb_source_id ASC`
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM edb_source WHERE 1=1 %s %s`, fields, condition, order)
	_, err = o.Raw(sql, pars).QueryRows(&items)
	return
}

// GetEdbSourceItemByCondition 获取指标来源
func GetEdbSourceItemByCondition(condition string, pars []interface{}) (item *EdbSource, err error) {
	o := orm.NewOrmUsingDB("data")
	sql := fmt.Sprintf(`SELECT * FROM edb_source WHERE 1=1 %s`, condition)
	err = o.Raw(sql, pars).QueryRow(&item)
	return
}

// InitEdbSourceVar 初始化时加载指标来源对应信息, 避免循环中查库, 注意edb_source表修改table_name的话需要重启服务
func InitEdbSourceVar() {
	EdbDataTableNameMap = make(map[int]string)
	EdbDataRefreshMethodMap = make(map[int]string)
	EdbTableNameSourceMap = make(map[string]*EdbSource)
	EdbSourceIdMap = make(map[int]*EdbSource)
	EdbSourceExtendIdMap = make(map[string]int)
	sources, e := GetEdbSourceItemsByCondition(``, make([]interface{}, 0), []string{}, "")
	if e != nil {
		utils.FileLog.Info("init source table err: %s", e.Error())
		return
	}
	for _, v := range sources {
		EdbDataTableNameMap[v.EdbSourceId] = v.TableName
		EdbDataRefreshMethodMap[v.EdbSourceId] = v.EdbRefreshMethod
		EdbTableNameSourceMap[v.TableName] = v

		EdbSourceIdMap[v.EdbSourceId] = v
		if v.SourceExtend != "" {
			arr := strings.Split(v.SourceExtend, ",")
			if len(arr) == 0 {
				continue
			}
			for _, s := range arr {
				EdbSourceExtendIdMap[s] = v.EdbSourceId
			}
		}
	}
}
